use rusqlite::*;

const DBPATH: &'static str = "data/tracker.db";
const STATEMENTS: [&str; 2] = [
"CREATE TABLE log (id integer primary key autoincrement, time text, latitude float, longitude float);",
"CREATE INDEX idx_time on log (time);",];
const QUERY_INSERT: &'static str =
    "INSERT INTO log (time, latitude, longitude) values (:time, :latitude, :longitude)";

pub fn connectdb() -> Result<Connection> {
    let conn = Connection::open(DBPATH)?;
    Ok(conn)
}

pub fn initdb(conn: &Connection) -> Result<()> {
    create_tables(&conn)?;
    set_pragmas(&conn)?;
    Ok(())
}

fn create_tables(conn: &Connection) -> Result<()> {
    for s in STATEMENTS {
        match conn.execute(s, params![]) {
            Ok(_) => {}
            Err(err) => println!("update failed: {}", err),
        }
    }
    Ok(())
}

fn set_pragmas(conn: &Connection) -> Result<()> {
    conn.pragma_update(Some(DatabaseName::Main), "journal_mode", "WAL")?;
    Ok(())
}

pub fn insert(conn: &Connection, time: &String, latitude: &f64, longitude: &f64) -> Result<()> {
    match conn.execute(QUERY_INSERT, params![time, latitude, longitude]) {
        Ok(inserted) => println!("{} rows were inserted", inserted),
        Err(err) => println!("insert failed: {}", err),
    }
    Ok(())
}
